{% extends "!layout.html" %}

{% block htmltitle %}
  {{ super() }}
  <!-- htmltitle is before nature.css - we use this hack to load bootstrap first -->
  <meta name="viewport" content="width=device-width, initial-scale=1.0" />
  <link rel="stylesheet" href="{{ pathto('_static/css/bootstrap.min.css', 1) }}" media="screen" />
  <link rel="stylesheet" href="{{ pathto('_static/css/bootstrap-responsive.css', 1) }}"/>
{% endblock %}

<!-- sidebar -->
{% block extrahead %}
  {% if pagename != 'index' %}
   {% if theme_collapsiblesidebar|tobool %}
       <script type="text/javascript" src="{{ pathto('_static/sidebar.js', 1) }}"></script>
   {% endif %}
  {% endif %}
  <meta name="viewport" content="width=device-width, initial-scale=1.0" />
  <script src="{{ pathto('_static/js/bootstrap.min.js', 1)}}" type="text/javascript"></script>
  <!-- <link rel="canonical" href="https://bioproj.extra.cea.fr/redmine/projects/nsap/{{pagename}}.html" /> -->

  <script type="text/javascript">
    $("div.buttonNext, div.buttonPrevious").hover(
       function () {
           $(this).css('background-color', '#FF9C34');
       },
       function () {
           $(this).css('background-color', '#A7D6E2');
       }
    );
    var bodywrapper = $('.bodywrapper');
    var sidebarbutton = $('#sidebarbutton');
    sidebarbutton.css({'height': '900px'});
  </script>
{% endblock %}

<!-- page title -->
    {% set title = 'SOMA-Pipeline' %}

<!-- header -->
{% block header %}

<div class="header-wrapper">
  <div class="header">
    <p class="logo">
      <a href="{{ pathto('index') }}">
        <img src="{{ pathto("_static/soma.png", 1) }}" alt="Logo"/>
      </a>
    </p>

    {%- block navbar -%}
    <div class="navbar">
      <ul>
        <li><a href="{{pathto('index')}}">Home</a></li>
        <li><a href="{{pathto('installation')}}">Installation</a></li>
        <li class="btn-li">
          <div class="btn-group">
		    <a href="{{pathto('documentation')}}">Documentation</a>
		    <a class="btn dropdown-toggle" data-toggle="dropdown">
		      <span class="caret"></span>
		    </a>
		    <ul class="dropdown-menu">
	          <li class="link-title">SOMA-Pipeline</li>
			  <li><a href="{{ pathto('user_guide_tree/index') }}">User guide</a></li>
			  <li><a href="{{ pathto('api_tree/index') }}">API</a></li>
			  <li class="divider"></li>
		      <li><a href="{{ pathto('developer_tree/index') }}">Development</a></li>
		    </ul>
		  </div>
        </li>
        <li><a href="http://neurospin-wiki.org/">NeuroSpin Wiki</a></li>
      </ul>
    </div> <!-- end navbar -->
    {%- endblock -%}
  </div>
</div>

{%- if pagename == 'index' %}
<!-- Banner -->
<div class="container banner-container">
  <div class="row-fluid banner-inner">
    <div class="hidden-phone">
      <div class="span6">
	    <div class="row-fluid">
          <div class="offset2 span8"><div id="index_carousel_tn" class="thumbnail">
	        <div id="examples_carousel" class="carousel slide" data-interval="false">
		      <ol id="nsap_index_indicators" class="carousel-indicators">
		        <li data-target="#examples_carousel" data-slide-to="0" class="active"></li>
		        <li data-target="#examples_carousel" data-slide-to="1"></li>
		        <li data-target="#examples_carousel" data-slide-to="2"></li>
		      </ol>
		      <!-- Carousel items -->
		      <div class="carousel-inner">
		        <div class="active item">
		          <a href="{{pathto('index')}}">
		            <div class="crop-wrapper" style="width: 550px; height: 200px; overflow: hidden">
			          <img src="{{ pathto("_static/clustering.gif", 1) }}"
			         style="max-height: 200px; max-width: 629px; margin-left: 60px;">
                    </div>
                  </a>
		        </div>
		        <div class="item">
		          <a href="{{pathto('index')}}">
		            <img src="{{ pathto("_static/soma.png", 1) }}">
                  </a>
		        </div>
		        <div class="item">
		          <a href="{{pathto('index')}}">
		            <img src="{{ pathto("_static/soma.png", 1) }}">
                  </a>
		        </div>
		      </div>
		      <!-- Carousel nav -->
		      <a class="carousel-control left" href="#examples_carousel" data-slide="prev">&lsaquo;</a>
		      <a class="carousel-control right" href="#examples_carousel" data-slide="next">&rsaquo;</a>
	        </div>
	      </div>
        </div>
	  </div>
    </div>
  </div>

  <div id="nsap_synopsis" class="span6">
    <h1>SOMA</h1>
    <h2>Pipeline</h2>
    <ul>
      <li>Simple and efficient tools for medical imaging analysis</li>
      <li>Accessible to everybody, and reusable in various contexts</li>
      <li>Built on Nipype Interfaces</li>
     </ul>
   </div>
  </div>
</div>
{%- endif %}

{% endblock %}


<!-- body -->
{% block content %}
<div class="content-wrapper">

  {%- if pagename != 'documentation' %}
    <div class="sphinxsidebar">
      <div class="sphinxsidebarwrapper">

        <!-- info setup -->
        {%- if pagename != "installation" %}
          <p class="doc-version">
           This documentation is for {{project}} <strong>version {{ release|e }}</strong>
          </p>
        {%- endif %}
        <p class="citing">
          If you use the software, please do not hesitate to 
          <a &mdash; <a href="https://bioproj.extra.cea.fr/redmine/projects/brainvisa-soma/">
          Report a Bug</a>.
        </p>
        <!-- <form action="/scripts/nsap-src-message.php" method="post">
          <textarea rows="4" name="comments" style="width:175px">Enter here your comments</textarea>
          <center>
           <input type="submit" value="Send">
          </center>
        </form> -->

      <!-- toc tree -->
      {{ toc }}

      </div>
    </div>
  {% endif %}

  <div class="content">
    {%- block document %}
      {{ super() }}
    {%- endblock %}
    <div class="clearer">
    </div>
  </div>
  
</div>
{% endblock %}

{% block relbar1 %}{% endblock %}
{% block relbar2 %}{% endblock %}


<!-- footer -->
{%- block footer %}
<div class="footer">
{%- if show_copyright %}
  {%- if hasdoc('copyright') %}
    {% trans path=pathto('copyright'), copyright=copyright|e %}&copy; {{ copyright }}.{% endtrans %}
  {%- else %}
    {% trans copyright=copyright|e %}&copy; {{ copyright }}.{% endtrans %}
  {%- endif %}
{%- endif %}
</div>
{%- endblock %}


